/* FreeRTOS related resources. */
#include "freertos_start.h"

/* Other includes. */
#include "r_cg_intc.h"
#include "r_cg_serial.h"
#include <string.h>

/* NOTE: Renesas FreeRTOS project does not support the main() function.
When you need it, try to use the Processing_Before_Start_Kernel() function
in the src/frtos_startup/freertos_start.c instead of it. */

void main_task(void *pvParameters)
{
    (void) pvParameters;

    while (1)
    {
        taskENTER_CRITICAL();
        P1_bit.no6 = !P1_bit.no6;
        taskEXIT_CRITICAL();
    }
}

void second_task(void *pvParameters)
{
    (void) pvParameters;

    while (1)
    {
        taskENTER_CRITICAL();
        P1_bit.no5 = !P1_bit.no5;
        taskEXIT_CRITICAL();
    }
}

void third_task(void *pvParameters)
{
    (void) pvParameters;

    while (1)
    {
        taskENTER_CRITICAL();
        P1_bit.no4 = !P1_bit.no4;
        taskEXIT_CRITICAL();
    }
}

#define LED_B P5_bit.no0
#define LED_R P1_bit.no7
#define LED_ON  0
#define LED_OFF 1
#define LAB_TIMEOUT_ERROR P2_bit.no2
#define LAB_FRAMING_ERROR P2_bit.no3
#define LAB_PARITY_ERROR  P2_bit.no4
#define LAB_OVERRUN_ERROR P2_bit.no5
#define LAB_SHOW 1
#define LAB_HIDE 0
#define LAB_CLEAR() do{ P2 &= 0xC3; } while (0)
#define UART_RECV_DEMO_SIZE 3

void uart_recv_demo_task(void *pvParameters)
{
    volatile uint8_t recv_buff[UART_RECV_DEMO_SIZE + 1];
    volatile uint8_t err_type;
    BaseType_t ret;

    (void) pvParameters;
    taskENTER_CRITICAL();
    LED_B = LED_ON;
    taskEXIT_CRITICAL();

    R_INTC3_Start();
    U_UART0_Start();

    for (;;)
    {
        /* --- Waiting for a click of button on the Simulator GUI --- */
        xSemaphoreTake( xSemaphoreINTP3, portMAX_DELAY );

        taskENTER_CRITICAL();
        LED_B = LED_OFF;
        LED_R = LED_OFF;
        LAB_CLEAR();
        taskEXIT_CRITICAL();
        memset( (char *)recv_buff, 0, UART_RECV_DEMO_SIZE + 1 );

        U_UART0_Receive_Start( recv_buff, UART_RECV_DEMO_SIZE, &err_type );

        /* --- Waiting for a transmission of serial data on the Simulator GUI --- */
        ret = xSemaphoreTake( xSemaphoreCBRECEIVEEND_CBERROR, pdMS_TO_TICKS(100) );
        if (pdPASS == ret && 0 == err_type)
        {
            taskENTER_CRITICAL();
            LED_B = LED_ON;
            vPrintString( "Received Message: " );
            vPrintString( (char *)recv_buff );
            vPrintString( "\n" );
            taskEXIT_CRITICAL();
        }
        else
        {
            taskENTER_CRITICAL();
            LED_R = LED_ON;
            if (pdFAIL == ret)
            {
                U_UART0_Receive_Stop();
                LAB_TIMEOUT_ERROR = LAB_SHOW;
                vPrintString( "Timeout Error\n" );
            }
            else
            {
                if (0x04 & err_type)
                {
                    LAB_FRAMING_ERROR = LAB_SHOW;
                    vPrintString( "Framing Error\n" );
                }
                if (0x02 & err_type)
                {
                    LAB_PARITY_ERROR = LAB_SHOW;
                    vPrintString( "Parity Error\n" );
                }
                if (0x01 & err_type)
                {
                    LAB_OVERRUN_ERROR = LAB_SHOW;
                    vPrintString( "Overrun Error\n" );
                }
            }
            taskEXIT_CRITICAL();
        }

        NOP();  /* for breakpoint, check timing chart on Simulator GUI */
    }
}
